@@ -236,6 +236,11 @@ impl<L, F, S> Filtered<L, F, S> {
 
     #[inline(always)]
     fn id(&self) -> FilterId {
+        debug_assert!(
+            !self.id.0.is_disabled(),
+            "a `Filtered` layer was used, but it had no `FilterId`; \
+            was it registered with the subscriber?"
+        );
         self.id.0
     }
 
@@ -404,7 +409,7 @@ where
 
 impl FilterId {
     const fn disabled() -> Self {
-        Self(u64::MAX)
+        Self(std::u64::MAX)
     }
 
     /// Returns a `FilterId` that will consider _all_ spans enabled.
@@ -504,6 +509,10 @@ impl FilterId {
 
         Self(self.0 | other)
     }
+
+    fn is_disabled(self) -> bool {
+        self.0 == Self::disabled().0
+    }
 }
 
 impl fmt::Debug for FilterId {
@@ -539,7 +548,7 @@ impl fmt::Binary for FilterId {
 
 impl FilterMap {
     pub(crate) fn set(self, FilterId(mask): FilterId, enabled: bool) -> Self {
-        if mask == u64::MAX {
+        if mask == std::u64::MAX {
             return self;
         }
 
@@ -561,7 +570,7 @@ impl FilterMap {
 
     #[inline]
     pub(crate) fn any_enabled(self) -> bool {
-        self.bits != u64::MAX
+        self.bits != std::u64::MAX
     }
 }
 
@@ -700,6 +709,22 @@ impl FilterState {
         }
     }
 
+    /// Clears the current in-progress filter state.
+    ///
+    /// This resets the [`FilterMap`] and current [`Interest`] as well as
+    /// clearing the debug counters.
+    pub(crate) fn clear_enabled() {
+        // Drop the `Result` returned by `try_with` --- if we are in the middle
+        // a panic and the thread-local has been torn down, that's fine, just
+        // ignore it ratehr than panicking.
+        let _ = FILTERING.try_with(|filtering| {
+            filtering.enabled.set(FilterMap::default());
+
+            #[cfg(debug_assertions)]
+            filtering.counters.in_filter_pass.set(0);
+        });
+    }
+
     pub(crate) fn take_interest() -> Option<Interest> {
         FILTERING
             .try_with(|filtering| {
@@ -718,8 +743,10 @@ impl FilterState {
     pub(crate) fn filter_map(&self) -> FilterMap {
         let map = self.enabled.get();
         #[cfg(debug_assertions)]
-        if self.counters.in_filter_pass.get() == 0 {
-            debug_assert_eq!(map, FilterMap::default());
+        {
+            if self.counters.in_filter_pass.get() == 0 {
+                debug_assert_eq!(map, FilterMap::default());
+            }
         }
 
         map
